-- Test company_geography
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(30);

set search_path to camper, public;

select has_table('company_geography');
select has_pk('company_geography');
select table_privs_are('company_geography', 'guest', array['SELECT']);
select table_privs_are('company_geography', 'employee', array['SELECT']);
select table_privs_are('company_geography', 'admin', array['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('company_geography', 'authenticator', array[]::text[]);

select has_column('company_geography', 'company_id');
select col_is_pk('company_geography', 'company_id');
select col_is_fk('company_geography', 'company_id');
select fk_ok('company_geography', 'company_id', 'company', 'company_id');
select col_type_is('company_geography', 'company_id', 'integer');
select col_not_null('company_geography', 'company_id');
select col_hasnt_default('company_geography', 'company_id');

select has_column('company_geography', 'geog');
select col_type_is('company_geography', 'geog', 'geography');
select col_not_null('company_geography', 'geog');
select col_hasnt_default('company_geography', 'geog');


set client_min_messages to warning;
truncate company_geography cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 'ca')
     , (8, 'Company 8', 'XX345', '', '777-777-777', 'c@c', '', '', '', '', '', 'DE', 'USD', 'en')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into company_geography (company_id, geog)
values (2, 'SRID=4326;POINT(-118.4079 33.9434)')
     , (4, 'SRID=4326;POINT(2.5559 49.0083)')
;

prepare geog_data as
select company_id, st_x(geog::geometry), st_y(geog::geometry)
from company_geography
;

set role guest;
select bag_eq(
	'geog_data',
	$$ values (2, -118.4079, 33.9434)
	        , (4,    2.5559, 49.0083)
	$$,
	'Everyone should be able to list all geography across all companies'
);
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');

select lives_ok(
	$$ delete from company_geography where company_id = 2 $$,
	'Admin from company 2 should be able to delete geography from that company.'
);

select bag_eq(
	'geog_data',
	$$ values (4, 2.5559, 49.0083)
	$$,
	'The row should have been deleted.'
);

select lives_ok(
	$$ insert into company_geography(company_id, geog) values (2, 'SRID=4326;POINT(-22.6056 63.9850)') $$,
	'Admin from company 2 should be able to insert a new geography to that company.'
);

select bag_eq(
	'geog_data',
	$$ values (2, -22.6056, 63.9850)
	        , (4,   2.5559, 49.0083)
	$$,
	'The new row should have been added'
);

select lives_ok(
	$$ update company_geography set geog = 'SRID=4326;POINT(139.7330 35.5670)' where company_id = 2 $$,
	'Admin from company 2 should be able to update geography of that company.'
);

select bag_eq(
	'geog_data',
	$$ values (2, 139.7330, 35.5670)
	        , (4,   2.5559, 49.0083)
	$$,
	'The row should have been updated.'
);


select throws_ok(
	$$ insert into company_geography (company_id, geog) values (8, 'SRID=4326;POINT(2.5559 49.0083)') $$,
	'42501', 'new row violates row-level security policy for table "company_geography"',
	'Admin from company 2 should NOT be able to insert new geography to another company 8.'
);

select lives_ok(
	$$ update company_geography set geog = 'SRID=4326;POINT(139.7330 35.5670)' where company_id = 4 $$,
	'Admin from company 2 should not be able to update geography of company 4, but no error if company_id is not changed.'
);

select bag_eq(
	'geog_data',
	$$ values (2, 139.7330, 35.5670)
	        , (4,   2.5559, 49.0083)
	$$,
	'No row should have been changed.'
);

select throws_ok(
	$$ update company_geography set company_id = 4 where company_id = 2 $$,
	'42501', 'new row violates row-level security policy for table "company_geography"',
	'Admin from company 2 should NOT be able to move geography to company 4'
);

select lives_ok(
	$$ delete from company_geography where company_id = 4 $$,
	'Admin from company 2 should NOT be able to delete geography from company 4, but not error is thrown'
);

select bag_eq(
	'geog_data',
	$$ values (2, 139.7330, 35.5670)
	        , (4,   2.5559, 49.0083)
	$$,
	'No row should have been changed'
);

reset role;


select *
from finish();

rollback;

